/* eslint-disable @typescript-eslint/no-namespace */
import type { SignupModalPressedTrigger } from '@holi/core/tracking/types'
import type { Channel } from '@holi/core/screens/notifications/hooks/useNotificationPreferences'

type RecommendationType = 'space' | 'donation' | 'volunteering_opportunity' | 'goodnews' | 'space_task'

/**
 * The base type of all tracking events. `TrackingEvent` instances shall not be defined outside of this file to ensure
 * this file contains a complete reference of all tracking events.
 */
export interface TrackingEvent {
  /** The event name */
  name: string

  /**
   * The events version (must use semantic versioning).
   *
   * As filtering by version in PostHog is not easily done based on a string, we're adopting the approach of the
   * semver-flattener plugin (https://github.com/PostHog/semver-flattener-plugin/) that splits the string up in multiple
   * number-based fields. These can be filtered and compared for using <, >, =, ... operators.
   */
  event_version__major: number
  event_version__minor: number
  event_version__patch: number

  /** Additional properties that can be added to the event payload */
  properties?: { [key: string]: unknown }
}

const versionOne = {
  event_version__major: 1,
  event_version__minor: 0,
  event_version__patch: 0,
}

/**
 * The `TrackingEvent` namespace contains all "custom tracking events" that can be sent to PostHog, thereby serving as
 * a *complete* reference. It contains sub-namespaces for the different areas of the application.
 */
export namespace TrackingEvent {
  export const SearchButtonPressed: TrackingEvent = {
    name: 'searchButtonPressed',
    ...versionOne,
  }

  export type SearchResult = {
    query: string
    totalNrOfResults: number
    totalNrOfPages: number
    currentPage: number
    selectedFacets: string[]
    // deliberately untyped to not have to adopt for every document type added. this shall only contain document
    // properties and not Instantsearch.js metadata on docs, see `removeDeniedProps` in
    // core/screens/search/typesense/tracking.ts
    renderedResults: Record<string, any>[] // eslint-disable-line @typescript-eslint/no-explicit-any
  }

  export const SearchResultPressed = ({
    itemPressed,
    itemPressedIndex,
    itemPressedType,
    itemPressedTitleDe,
    itemPressedTitleEn,
    searchResult,
  }: {
    itemPressed: Record<string, any> // eslint-disable-line @typescript-eslint/no-explicit-any
    itemPressedIndex: number
    itemPressedType: string
    itemPressedTitleDe?: string
    itemPressedTitleEn?: string
    searchResult: SearchResult
  }): TrackingEvent => ({
    name: 'searchResultPressed',
    ...versionOne,
    properties: {
      itemPressed,
      itemPressedIndex,
      itemPressedType,
      itemPressedTitleDe,
      itemPressedTitleEn,
      ...searchResult,
    },
  })

  export const SearchResultsRendered = ({
    query,
    totalNrOfResults,
    totalNrOfPages,
    currentPage,
    selectedFacets,
    renderedResults,
  }: SearchResult): TrackingEvent => ({
    name: 'searchResultsRendered',
    ...versionOne,
    properties: {
      query,
      totalNrOfResults,
      totalNrOfPages,
      currentPage,
      renderedResults,
      selectedFacets,
    },
  })

  export const SpaceAskQuestion = {
    askedQuestion: {
      name: 'askQuestionButton_pressed',
      ...versionOne,
    },
    startedConversation: {
      name: 'startConversationButton_pressed',
      ...versionOne,
    },
  }

  export const Engaging = {
    reacted: (reactableType: string, reactableId: string): TrackingEvent => ({
      name: 'reacted',
      properties: {
        reactableType,
        reactableId,
      },
      ...versionOne,
    }),
    commented: (commentableType: string, commentableId: string): TrackingEvent => ({
      name: 'commented',
      properties: {
        commentableType,
        commentableId,
      },
      ...versionOne,
    }),
  }

  export const FeedPosts = {
    read: (postId: string): TrackingEvent => ({
      name: 'readFeedPost',
      properties: {
        postId,
      },
      ...versionOne,
    }),
    shared: (postId: string): TrackingEvent => ({
      name: 'sharedFeedPost',
      properties: {
        postId,
      },
      ...versionOne,
    }),
    feedPostCreateStart: {
      name: 'feedPostCreateStart',
      ...versionOne,
    },
    viewSummary: (
      totalElements: number,
      feedPosts: number,
      spacePosts: number,
      goodNews: number,
      insights: number,
      volunteering: number
    ): TrackingEvent => ({
      name: 'feedViewSummary',
      properties: { totalElements, feedPosts, spacePosts, goodNews, insights, volunteering },
      ...versionOne,
    }),
  }

  export const Spaces = {
    followSpaceButtonPressed: (spaceId: string, isLoggedIn: boolean): TrackingEvent => ({
      name: 'followSpaceButtonPressed',
      properties: { spaceId, isLoggedIn },
      ...versionOne,
    }),

    joinSpaceButtonPressed: (spaceId: string, isLoggedIn: boolean): TrackingEvent => ({
      name: 'joinSpaceButtonPressed',
      properties: { spaceId, isLoggedIn },
      ...versionOne,
    }),

    FileStorage: {
      accessed: (spaceId: string): TrackingEvent => ({
        name: 'spaceFileStorageAccessed',
        properties: { spaceId },
        ...versionOne,
      }),
    },

    LinkHub: {
      linkAdded: (spaceId: string, linkType: string): TrackingEvent => ({
        name: 'spaceLinkHubLinkAdded',
        properties: { spaceId, linkType },
        ...versionOne,
      }),
    },

    Onboarding: {
      updated: (spaceId: string): TrackingEvent => ({
        name: 'spaceOnboardingUpdated',
        properties: { spaceId },
        ...versionOne,
      }),
    },

    Profile: {
      externalLinkAdded: (spaceId: string): TrackingEvent => ({
        name: 'spaceProfileExternalLinkAdded',
        properties: { spaceId },
        ...versionOne,
      }),
    },
  }

  export const Donations = {
    pressed: (projectId: number): TrackingEvent => ({
      name: 'pressedDonateButton',
      properties: { projectId },
      ...versionOne,
    }),
  }

  export const Volunteering = {
    pressed: (engagementId: string): TrackingEvent => ({
      name: 'pressedVolunteeringEngagementButton',
      properties: { engagementId },
      ...versionOne,
    }),
  }

  export const SpacePosts = {
    read: (postId: string, spaceId?: string): TrackingEvent => ({
      name: 'readSpacePost',
      properties: {
        spaceId,
        postId,
      },
      ...versionOne,
    }),
  }

  export namespace Auth {
    // Others
    export const RegisterButtonPressed: TrackingEvent = {
      name: 'registerButtonPressed',
      ...versionOne,
    }

    export const LoginButtonPressed: TrackingEvent = {
      name: 'loginButtonPressed',
      ...versionOne,
    }
    export const ContinueAsGuestButtonPressed: TrackingEvent = {
      name: 'continueAsGuestButtonPressed',
      ...versionOne,
    }
    export const ForgotPasswordButtonPressed: TrackingEvent = {
      name: 'forgotPasswordButtonPressed',
      ...versionOne,
    }
    export const ActivateAccountButtonPressed: TrackingEvent = {
      name: 'activateAccountButtonPressed',
      ...versionOne,
    }
    export const ResendEmailButtonPressed: TrackingEvent = {
      name: 'resendEmailButtonPressed',
      ...versionOne,
    }
    export const OAuthWithGoogleButtonPressed: TrackingEvent = {
      name: 'oauthWithGoogleButtonPressed',
      ...versionOne,
    }
    export const OAuthWithFacebookButtonPressed: TrackingEvent = {
      name: 'oauthWithFacebookButtonPressed',
      ...versionOne,
    }
  }

  export const ContentTranslated = (
    type: 'feed_post' | 'space_post' | 'comment' | 'reply',
    contentId: string
  ): TrackingEvent => ({
    name: 'content_translated',
    ...versionOne,
    properties: {
      type: type,
      contentId: contentId,
    },
  })

  export const RecommendationPressed = (type: RecommendationType, itemId: string, index: number): TrackingEvent => ({
    name: 'recommendation_pressed',
    ...versionOne,
    properties: {
      type: type,
      itemId: itemId,
      index: index,
    },
  })

  export const RecommendationViewed = (
    type: RecommendationType | 'challenge',
    otherProperties: Record<string, string | number>
  ): TrackingEvent => ({
    ...versionOne,
    name: 'recommendation_viewed',
    properties: {
      type,
      ...otherProperties,
    },
  })

  export namespace InsightCard {
    export const CardPressed = (insightId: string): TrackingEvent => ({
      name: 'insightCard_pressed',
      properties: {
        insightId: insightId,
      },
      ...versionOne,
    })
    export const CardViewed = (insightId: string): TrackingEvent => ({
      name: 'insightCard_viewed',
      properties: {
        insightId: insightId,
      },
      ...versionOne,
    })
  }

  export namespace ChallengeCard {
    export const CardPressed = (challengeId: string, index: number): TrackingEvent => ({
      name: 'challengeCard_pressed',
      properties: {
        challengeId: challengeId,
        index: index,
      },
      ...versionOne,
    })

    export const SubmissionPressed = (
      challengeId: string,
      submissionId: string,
      submissionIndex: number
    ): TrackingEvent => ({
      name: 'challengeCardSubmission_pressed',
      properties: {
        challengeId: challengeId,
        submissionId: submissionId,
        submissionIndex: submissionIndex,
      },
      ...versionOne,
    })
  }

  export const OnboardingStepCompleted = (
    step: 'welcome' | 'contentSelection' | 'locationSelection' | 'interestsSelection' | 'skillsSelection',
    answers: string[],
    buttonPressed: 'continue' | 'skip' | 'back'
  ): TrackingEvent => ({
    name: 'onboardingStepCompleted',
    properties: {
      step,
      answers,
      buttonPressed,
    },
    ...versionOne,
  })

  export const SignupStepCompleted = (
    properties:
      | {
          step: 'emailAndPassword'
          answer: 'signUp' | 'continueAsGuest' | 'login'
        }
      | {
          step: 'verificationCode'
          answer: 'verifyAccount' | 'continueAsGuest'
        }
      | { step: 'name'; answer: 'continue' }
  ): TrackingEvent => ({
    name: 'signupStepCompleted',
    properties,
    ...versionOne,
  })

  export const SignUpModalPressed = (trigger: SignupModalPressedTrigger, answer: string): TrackingEvent => ({
    name: 'signUpModalPressed',
    properties: { trigger, answer },
    ...versionOne,
  })

  export namespace Chat {
    interface ChatMessageMetadataBase {
      roomId: string
      content: 'text' | 'file' | 'image'
    }

    interface ChatUserMessageMetadata extends ChatMessageMetadataBase {
      type: 'direct_message'
    }

    interface ChatSpaceMessageMetadata extends ChatMessageMetadataBase {
      type: 'space_message'
      spaceId: string
      spaceName: string
    }

    type ChatMessageMetadata = ChatUserMessageMetadata | ChatSpaceMessageMetadata

    export const MessageSent = (metadata: ChatMessageMetadata): TrackingEvent => ({
      name: 'chatMessageSent',
      properties: {
        ...metadata,
      },
      ...versionOne,
    })
  }

  export const NotYetAvailableFeaturePressed = (feature: string): TrackingEvent => ({
    name: 'notYetAvailableFeaturePressed',
    properties: { feature },
    ...versionOne,
  })

  export const UserConnectionRequested = (requesterId: string, recipientId: string): TrackingEvent => ({
    name: 'userConnectionRequested',
    properties: { requesterId, recipientId },
    ...versionOne,
  })

  export const UserConnectionAccepted = (requesterId: string, recipientId: string): TrackingEvent => ({
    name: 'userConnectionAccepted',
    properties: { requesterId, recipientId },
    ...versionOne,
  })

  export const UserConnectionRejected = (requesterId: string, recipientId: string): TrackingEvent => ({
    name: 'userConnectionRejected',
    properties: { requesterId, recipientId },
    ...versionOne,
  })
  export const PollVoteSubmitted = (pollId: string): TrackingEvent => ({
    name: 'pollVoteSubmitted',
    properties: { pollId },
    ...versionOne,
  })

  export const UserProfile = {
    editPressed: (): TrackingEvent => ({
      name: 'userProfileEdit_pressed',

      ...versionOne,
    }),
    completionDismissed: (page: 'profile' | 'home_feed'): TrackingEvent => ({
      name: 'userProfileCompletion_dismissed',
      properties: {
        page: page,
      },
      ...versionOne,
    }),
  }

  export const NotificationPreferenceChanged = (
    workflowId: string,
    workflowName: string,
    channel: Channel,
    newState: boolean
  ): TrackingEvent => ({
    ...versionOne,
    name: 'notificationPreferenceChanged',
    properties: {
      workflowId,
      channel,
      newState,
    },
  })
}

export const Unknown: TrackingEvent = {
  name: 'unknown',
  ...versionOne,
}
